#!/usr/bin/env python
################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
################################################################################
# glite-wms-config v. 2.3.5
#
# Post-installation script for configuring the gLite Workload Management System
# Robert Harakaly < robert.harakaly@cern.ch >
# Leanne Guy <leanne.guy@cern.ch>
#
# Version info: $Id: glite-wms-config.py,v 1.107.2.26 2006/10/05 11:55:12 harakaly Exp $
#
# Usage: python glite-wms-config [-c|-v|-h|--help|--start|--stop|--startservice=<service name>|--stopservice=<service name>]
#        -c                               print configuration
#        -v                               print version
#        -h,--help                        print usage info
#        --configure                      configure the WMS node
#        --start                          start all services
#        --stop                           stop all services
#        --startservice=<service name>    start the named service
#        --stopservice=<service name>     stop the named service
#                                         Valid service names are:
#                                         condor = the Condor master and daemons
#                                         ftpd   = the Grid FTP daemon
#                                         wmp    = the gLite WMS Proxy Service
#                                         lbp    = the gLite LB Proxy Service
#                                         lm     = the gLite WMS Logger Monitor daemon
#                                         wm     = the gLite WMS Workload Manager daemon
#                                         ns     = the gLite WMS Network Server daemon
#                                         jc     = the gLite WMS Job Controller daemon
#                                         pr     = the gLite WMS Proxy Renewal daemon
#                                         lb     = the gLite WMS Logging & Bookkeeping client
#
# Return codes: 0 - Ok
#               1 - Failed
#
################################################################################

# Additional instructions

import os,string,re,pwd,time
import os.path
from shutil import copy
import sys, posix
import getopt
import xml.dom.minidom
from xml.dom.minidom import getDOMImplementation
import _xmlplus.xpath as xpath
from xml.dom.ext import PrettyPrint

sys.path.append(".")
from gLiteInstallerLib import gLib as GLib
from  gLiteInstallerLib import ConfigParams
from gLiteInstallerLib import ExportFile
from gLiteInstallerLib import ClassAdsParser
from gridSecurity import gridSecurity
from globus import Globus
from ftpd import Ftpd
from condor import Condor
import mysql as MySQL
from serviceDiscovery import ServiceDiscovery
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global config
global nodeType
global verbose
global certs
global rgmaServicetool
global glib


class glite_wms:
    def __init__(self,params):
        self.params = params
        self.verbose = 0
        self.ftp = Ftpd(params)
        self.mysql = MySQL.Mysql()
        self.version = "2.3.5"
        self.name = "glite-wms"
        self.friendly_name = "gLite Workload Management System"
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "       Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc/config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print('        -c                               print configuration')
        print('        -v                               print version')
        print('        -h,--help                        print usage info')
        print('        --configure                      configure the WMS services')
        print('        --start                          start all services')
        print('        --stop                           stop all services')
        print('        --status                         check status of all services')
        print('        --startservice=<service name>    start the named service')
        print('        --stopservice=<service name>     stop the named service')
        print('                                         Valid service names are:')
        print('                                         condor = the Condor master and daemons')
        print('                                         ftpd   = the Grid FTP daemon')
        print('                                         wmp    = the gLite WMS Proxy Service')
        print('                                         lbp    = the gLite LB Proxy Service')
        print('                                         lm     = the gLite WMS Logger Monitor daemon')
        print('                                         wm     = the gLite WMS Workload Manager daemon')
        print('                                         ns     = the gLite WMS Network Server daemon')
        print('                                         jc     = the gLite WMS Job Controller daemon')
        print('                                         pr     = the gLite WMS Proxy Renewal daemon')
        print('                                         lb     = the gLite WMS Logging & Bookkeeping client')
        print '\n'
    
    #-------------------------------------------------------------------------------
    # Start the service
    #-------------------------------------------------------------------------------

    def start(self,servicename='all'):

        if len(os.listdir(params['X509_VOMS_DIR'])) == 0:
            print("\n\n")
            glib.printErrorMessage("No VOMS Server host certificates could be found in %s" % params['X509_VOMS_DIR'])
            glib.printErrorMessage("Before using the %s," % service.friendly_name)
            glib.printErrorMessage("make sure that the host certificate of each used VOMS")
            glib.printErrorMessage("Server is installed in the %s directory." % params['X509_VOMS_DIR'])
            return 1

        #-------------------------------------------------------------------
        # MySQL
        #-------------------------------------------------------------------

        if servicename == 'all':
          pid = glib.getPID('mysqld_safe')
          if pid == 0:
            print "Starting MySQL daemon                                     ",
            errorcode = os.system("/usr/bin/mysqld_safe --datadir=/var/lib/mysql --pid=/var/lib/mysql/%s.pid --max_allowed_packet=%s  &" \
                                   % (glib.fq_hostname,params['mysql.max_allowed_packet']))
            time.sleep(5)
            if errorcode:
                glib.printFailedMessage()
                return errorcode
            else:
                glib.printOkMessage()
    
            if not os.path.exists('/tmp/mysql.sock'):
                os.symlink('/var/lib/mysql/mysql.sock', '/tmp/mysql.sock')
            
        #-------------------------------------------------------------------
        # Condor
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'condor':
            pid = glib.getPID('condor_master')
            if pid != 0:
                glib.printInfoMessage('The Condor master is already running. Restarting...')
                condor = Condor(params)
                condor.killName('condorc-launcher')
                condor.killName('condorc-advertiser')
                condor.stop()
            else:
                print "Starting Condor..."
    
            cmd = 'su - %s -c "export CONDOR_CONFIG=%s; %s"' % \
                (os.environ['GLITE_USER'], \
                os.environ['CONDOR_CONFIG'],"%s/sbin/condor_master" \
                % os.environ['CONDORG_INSTALL_PATH'])
            os.system(cmd)
        
            pid = glib.getPID('condor_master')
            if pid == 0:
                glib.printErrorMessage("Could not start Condor")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1
            else:
                print "The Condor master has been started                        ",
                glib.printOkMessage()

        #-------------------------------------------------------------------
        # Grid FTP
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'ftpd':
            pid = glib.getPID('ftpd')
            if pid != 0:
                print 'The Grid FTP Daemon is already running. Restarting...'
                self.ftp.stop()
            else:
                print "Starting the Grid FTP Daemon..."
    
            self.ftp.start()
    
            pid = glib.getPID('ftpd')
            if pid == 0:
                glib.printErrorMessage("Could not start the Grid FTP Daemon")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1
            else:
                print "The Grid FTP Daemon has been started                      ",
                glib.printOkMessage()

        #-------------------------------------------------------------------
        # local logger
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'lb':
            pid1 = glib.getPID('glite-lb-logd')
            pid2 = glib.getPID('glite-lb-interlogd')
            if (pid1 != 0) or (pid2 != 0):
                print 'The gLite LB Local Logger daemons are already running. Restarting...'
                os.system("%s/etc/init.d/glite-lb-locallogger stop" % os.environ['GLITE_LOCATION'])
            else:
                print "Starting the gLite LB Local Logger daemons"
        
            cmd = "%s/etc/init.d/glite-lb-locallogger start" % os.environ['GLITE_LOCATION']
            os.system(cmd)
        
            # Check that the daemon is running
            pid1 = glib.getPID('glite-lb-logd')
            pid2 = glib.getPID('glite-lb-interlogd')
            if (pid1 != 0) and (pid2 != 0):
                print "gLite LB Local Logger has been started                    ",
                glib.printOkMessage()
                print "gLite LB Interlogger has been started                     ",
                glib.printOkMessage()
            else:
                glib.printErrorMessage("Could not start the gLite LB Local Logger daemons")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1

        #-------------------------------------------------------------------
        # Log Monitor
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'lm':
            pid = glib.getPID('glite-wms-log_monitor')
            if pid != 0:
                print 'The gLite WMS Log Monitor is already running. Restarting...'
                os.system("%s/etc/init.d/glite-wms-lm stop" % os.environ['GLITE_LOCATION'])
            else:
                print "Starting the gLite WMS Log Monitor..."

            if os.path.exists("%s/logmonitor/lock" % os.environ["GLITE_LOCATION_VAR"]):
                os.remove("%s/logmonitor/lock" % os.environ["GLITE_LOCATION_VAR"])
            cmd = "%s/etc/init.d/glite-wms-lm start" % os.environ['GLITE_LOCATION']
            os.system(cmd)
    
            pid = glib.getPID('glite-wms-log_monitor')
            if pid == 0:
                glib.printErrorMessage("Could not start the gLite WMS Log Monitor")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1
            else:
                print "The gLite WMS Log Monitor has been started                ",
                glib.printOkMessage()

        #-------------------------------------------------------------------
        # Workload Manager 
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'wm':
            pid = glib.getPID('glite-wms-workload_manager')
            if pid != 0:
                print 'The gLite WMS Workload Manager is already running. Restarting...'
                os.system("%s/etc/init.d/glite-wms-wm stop" % os.environ['GLITE_LOCATION'])
            else:
                print "Starting the gLite WMS Workload Manager..."
    
            cmd = "%s/etc/init.d/glite-wms-wm start" % os.environ['GLITE_LOCATION']
            os.system(cmd)
        
            pid = glib.getPID('glite-wms-workload_manager')
            if pid == 0:
                glib.printErrorMessage("Could not start the gLite WMS Workload Manager")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1
            else:
                print "The gLite WMS Workload Manager has been started           ",
                glib.printOkMessage()

        #-------------------------------------------------------------------
        # Network Server
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'ns':
            pid = glib.getPID('glite-wms-manager_ns_daemon')
            if pid != 0:
                print 'The gLite WMS Network Server is already running. Restarting...'
                os.system("%s/etc/init.d/glite-wms-ns stop" % os.environ['GLITE_LOCATION'])
            else:
                print "Starting the gLite WMS Network Server..."
    
            cmd = "%s/etc/init.d/glite-wms-ns start" % os.environ['GLITE_LOCATION']
            os.system(cmd)
        
            pid = glib.getPID('glite-wms-manager_ns_daemon')
            if pid == 0:
                glib.printErrorMessage("Could not start the gLite WMS Network Server")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1
            else:
                print "The gLite WMS Network Server has been started             ",
                glib.printOkMessage()

        #-------------------------------------------------------------------
        # Job Controller
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'jc':
            pid = glib.getPID('glite-wms-job_controller')
            if pid != 0:
                print 'The gLite WMS Job Controller is already running. Restarting...'
                os.system("%s/etc/init.d/glite-wms-jc stop" % os.environ['GLITE_LOCATION'])
            else:
                print "Starting the gLite WMS Job Controller..."
    
            cmd = "%s/etc/init.d/glite-wms-jc start" % os.environ['GLITE_LOCATION']
            os.system(cmd)
    
            pid = glib.getPID('glite-wms-job_controller')
            if pid == 0:
                glib.printErrorMessage("Could not start the gLite gLite WMS Job Controller")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1
            else:
                print "The gLite WMS Job Controller has been started             ",
                glib.printOkMessage()

        #-------------------------------------------------------------------
        # Proxy Renewal
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'pr':
            pid = glib.getPID('glite-proxy-renewd')
            if pid != 0:
                print 'The gLite WMS Proxy Renewal Daemon is already running. Restarting...'
                os.system("%s/etc/init.d/glite-proxy-renewald stop" % os.environ['GLITE_LOCATION'])
                print "\n"
            else:
                print "Starting the gLite WMS Proxy Renewal Daemon..."
    
            cmd = "%s/etc/init.d/glite-proxy-renewald start" % os.environ['GLITE_LOCATION']
            os.system(cmd)
            print "\n"
        
            pid = glib.getPID('glite-proxy-renewd')
            if pid == 0:
                glib.printErrorMessage("Could not start the gLite WMS Proxy Renewal Daemon")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1
            else:
                print "The gLite WMS Proxy Renewal Daemon has been started       ",
                glib.printOkMessage()
        
        #-------------------------------------------------------------------
        # Initialize Condor
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'condor':
            # Start condorc-launcher and condorc-advertiser processes
            os.system('su - %s -c "condorc-initialize"' % \
                ( os.environ['GLITE_USER']))
        
        #-------------------------------------------------------------------
        # WMProxy
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'wmp':

            retval = os.system("/sbin/service httpd status")
            if retval == 0:
                print 'The gLite WM Proxy Service is already running. Restarting...'
                os.system("/sbin/service httpd stop")
                print "\n"
            else:
                print "Starting the gLite WM Proxy Service..."
            
            retval = os.system("/usr/sbin/httpd -k start -f %s/etc/glite_wms_wmproxy_httpd.conf" % os.environ['GLITE_LOCATION'])

            if retval != 0:
                glib.printErrorMessage("Could not start the gLite WM Proxy Service")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1
            else:
                print "The gLite WM Proxy Service has been started              ",
                glib.printOkMessage()
                
        #-------------------------------------------------------------------
        # LBProxy
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'lbp':

            retval = os.system("%s/etc/init.d/glite-lb-proxy status" % os.environ['GLITE_LOCATION'])
            if retval == 0:
                print 'The gLite LB Proxy Service is already running. Restarting...'
                os.system("%s/etc/init.d/glite-lb-proxy stop" % os.environ['GLITE_LOCATION'])
                print "\n"
            else:
                print "Starting the gLite LB Proxy Service..."
            
            retval = os.system("%s/etc/init.d/glite-lb-proxy start" % os.environ['GLITE_LOCATION'])

            if retval != 0:
                glib.printErrorMessage("Could not start the gLite LB Proxy Service")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1
            else:
                print "The gLite LB Proxy Service has been started              ",
                glib.printOkMessage()
                
        #-------------------------------------------------------------------
        # Start Servicetool
        #-------------------------------------------------------------------
    
        if servicename == 'all':
    
            if params['rgma.servicetool.activate'] == "true":
            
                errorcode = rgmaServicetool.start()
                if (errorcode != 0):
                    return 1
            
        if servicename == 'all':
            # Restart check cron job
            crontab = "/etc/cron.d/glite-wms-check-daemons.cron"
            cronjob = "*/5 * * * * root . /etc/glite/profile.d/glite_setenv.sh ; sh $GLITE_LOCATION/libexec/glite-wms-check-daemons.sh > /dev/null"
    
            if not os.path.exists(crontab):
                os.system("/bin/touch %s" % crontab)
            os.chmod(crontab,0644)
            f = open(crontab,'w')
            f.write("HOME=/\n")
            f.write("MAILTO=%s\n\n" % params['cron.mailto'])
            f.write(cronjob)
            f.write("\n")
            f.close()
            os.system("/bin/touch /var/spool/cron")
    
        return 0

    #-------------------------------------------------------------------------------
    # Service status
    #-------------------------------------------------------------------------------

    def status(self):
        
        error_level = 0

        #-------------------------------------------------------------------
        # Condor
        #-------------------------------------------------------------------

        retval = os.system('condor_status -any')
        if retval != 0:
            error_level = retval
        
        #-------------------------------------------------------------------
        # Grid FTP daemon
        #-------------------------------------------------------------------

        retval = self.ftp.status()
        if retval != 0:
            error_level = retval
    
        #-------------------------------------------------------------------
        # Workload Manager
        #-------------------------------------------------------------------

        retval = os.system("%s/etc/init.d/glite-wms-wm status" % os.environ['GLITE_LOCATION'])
        if retval != 0:
            error_level = retval

        #-------------------------------------------------------------------
        # Network Server
        #-------------------------------------------------------------------

        retval = os.system("%s/etc/init.d/glite-wms-ns status" % os.environ['GLITE_LOCATION'])
        if retval != 0:
            error_level = retval

        #-------------------------------------------------------------------
        # Job Controller
        #-------------------------------------------------------------------

        retval = os.system("%s/etc/init.d/glite-wms-jc status" % os.environ['GLITE_LOCATION'])
        if retval != 0:
            error_level = retval

        #-------------------------------------------------------------------
        # Log Monitor
        #-------------------------------------------------------------------

        retval = os.system("%s/etc/init.d/glite-wms-lm status" % os.environ['GLITE_LOCATION'])
        if retval != 0:
            error_level = retval

        #-------------------------------------------------------------------
        # Proxy Renewal
        #-------------------------------------------------------------------

        retval = os.system("%s/etc/init.d/glite-proxy-renewald status" % os.environ['GLITE_LOCATION'])
        if retval != 0:
            error_level = retval

        #-------------------------------------------------------------------
        # Local Logger
        #-------------------------------------------------------------------

        retval = os.system("%s/etc/init.d/glite-lb-locallogger status" % os.environ['GLITE_LOCATION'])
        if retval != 0:
            error_level = retval
        
        #-------------------------------------------------------------------
        # WMProxy
        #-------------------------------------------------------------------

        retval = os.system("/sbin/service httpd status")
        if retval != 0:
            error_level = retval
        
        #-------------------------------------------------------------------
        # LBProxy
        #-------------------------------------------------------------------

        retval = os.system("%s/etc/init.d/glite-lb-proxy status" % os.environ['GLITE_LOCATION'])
        if retval != 0:
            error_level = retval
        
        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
        
            if rgmaServicetool.status():
                error_level = 1
    
        return error_level
        
    #-------------------------------------------------------------------------------
    # Stop the services
    #-------------------------------------------------------------------------------

    def stop(self,servicename='all'):
        
        error_level = 0

        # Check if GLITE_USER has been changed without stopping the services
        uid,gid = glib.get_user_info(os.environ['GLITE_USER'])

        old_user = uid
        command = os.popen("ps auxn | grep glite-wms-log_monitor| grep -v 'grep glite-wms-log_monitor' |awk '{print $1}'")
        for line in command:
            old_user = line

        if int(old_user) != int(uid):
            glib.printErrorMessage("The glite.user.name parameter was modified in the config file without")
            glib.printErrorMessage("stopping the WMS services first or the user has been recreated with")
            glib.printErrorMessage("a different uid. This may cause inconsistencies in the")
            glib.printErrorMessage("running processes. It is recommended to stop the WMS services before")
            glib.printErrorMessage("modifying that parameter.")
            glib.printErrorMessage("Please stop all services manually before proceeding")
            return 1

        # Remove cron job
        if servicename == 'all':
            if os.path.exists("/etc/cron.d/glite-wms-check-daemons.cron"):
                os.remove("/etc/cron.d/glite-wms-check-daemons.cron")

            os.system("/bin/touch /var/spool/cron")
            
        #-------------------------------------------------------------------
        # LBProxy
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'lbp':
            if not os.system("%s/etc/init.d/glite-lb-proxy status" % os.environ['GLITE_LOCATION']):
                print "Stopping the LBProxy service                                ",
                
                retval = os.system("%s/etc/init.d/glite-lb-proxy stop" % os.environ['GLITE_LOCATION'])
                if retval != 0:
                    glib.printFailedMessage()
                    return 1
                    
                glib.printOkMessage()
                    
        #-------------------------------------------------------------------
        # MySQL
        #-------------------------------------------------------------------

        if servicename == 'all':
            self.mysql.stop()

        #-------------------------------------------------------------------
        # WMProxy
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'wmp':
            if not os.system("/sbin/service httpd status"):
                print "Stopping the WMProxy service                                ",
                
                retval = os.system("/sbin/service httpd stop")
                if retval != 0:
                    glib.printFailedMessage()
                    return 1
                    
                glib.printOkMessage()
                    
        #-------------------------------------------------------------------
        # Condor
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'condor':
            condor = Condor(params)
            condor.killName('condorc-launcher')
            condor.killName('condorc-advertiser')                 
            condor.killName('condorc-authorizer')                 
            condor.stop()

        #-------------------------------------------------------------------
        # Workload Manager
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'wm':
            retval = glib.stopProcess("glite-wms-workload_manager", "%s/etc/init.d/glite-wms-wm stop" % os.environ['GLITE_LOCATION'])
            if retval == 1:
                return 1
            elif retval ==2:
                error_level = 2

        #-------------------------------------------------------------------
        # Network Server
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'ns':
            retval = glib.stopProcess("glite-wms-manager_ns_daemon", "%s/etc/init.d/glite-wms-ns stop" % os.environ['GLITE_LOCATION'])
            if retval == 1:
                return 1
            elif retval ==2:
                error_level = 2

        #-------------------------------------------------------------------
        # Job Controller
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'jc':
            retval = glib.stopProcess("glite-wms-job_controller", "%s/etc/init.d/glite-wms-jc stop" % os.environ['GLITE_LOCATION'])
            if retval == 1:
                return 1
            elif retval ==2:
                error_level = 2

        #-------------------------------------------------------------------
        # Log Monitor
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'lm':
            retval = glib.stopProcess("glite-wms-log_monitor", "%s/etc/init.d/glite-wms-lm stop" % os.environ['GLITE_LOCATION'])
            if retval == 1:
                return 1
            elif retval ==2:
                error_level = 2

        #-------------------------------------------------------------------
        # Proxy Renewal
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'pr':
            retval = glib.stopProcess("glite-proxy-renewd", "%s/etc/init.d/glite-proxy-renewald stop" % os.environ['GLITE_LOCATION'])
            if retval == 1:
                return 1
            elif retval ==2:
                error_level = 2

        #-------------------------------------------------------------------
        # Local Logger
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'lb':
            retval = glib.stopProcess("glite-lb-logd", "%s/etc/init.d/glite-lb-locallogger stop" % os.environ['GLITE_LOCATION'])
            if retval == 1:
                return 1
            elif retval ==2:
                error_level = 2
        
        #-------------------------------------------------------------------
        # Grid FTP daemon
        #-------------------------------------------------------------------

        if servicename == 'all' or servicename == 'ftpd':
            pid = glib.getPID('ftpd')
            if pid == 0:
                print 'The Grid FTP Daemon was not running'
            else:
                print "Stopping the Grid FTP Daemon..."
                self.ftp.stop()
    
                pid = glib.getPID('ftpd')
                if pid != 0:
                    glib.printWarningMessage("Could not stop the Grid FTP Daemon")
                    error_level = 2
                else:
                    print "The Grid FTP Daemon has been stopped                      ",
                    glib.printOkMessage()

        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
        
            if rgmaServicetool.stop() != 0:
                error_level = 1
    
        return error_level
        
    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------

    def configure(self):

        error_level = 0

        #--------------------------------------------------------
        # Check group security
        #--------------------------------------------------------
        
        if not params.has_key('glite.user.group'):
            params['glite.user.group'] = params['glite.user.name']
        if params['glite.user.group'] == "":
            params['glite.user.group'] = params['glite.user.name']

        for vo in glib.getVoList():
            if params['glite.user.group'] == glib.getVoParams(vo)['pool.account.group']:
                glib.printErrorMessage("The parameters glite.user.group and pool.account.group for VO %s cannot have the same value" % vo)
                return 1
            
        #--------------------------------------------------------
        # Create glite user
        #--------------------------------------------------------

        if self.verbose:
            print "\nCreating gLite user %s...              " % params['glite.user.name']
        
        glib.add_user(params['glite.user.name'],params['glite.user.group']) 
        (uid,gid) = glib.get_user_info(params['glite.user.name'])

        if glib.checkInteractiveLogin("%s" % params['glite.user.name']):
            return 1

        #-------------------------------------------------------------------------------
        # VO list notice
        #-------------------------------------------------------------------------------
    
        print "\nThis WMS will be configured to serve the following VOs:"
        for vo in glib.getVoList():
            print "=> %s" % vo
            
        #-------------------------------------------------------------------------------
        # Update the $GLITE_LOCATION/etc/glite-mkgridmap.conf file if required
        #-------------------------------------------------------------------------------
    
        ret_val = gsec.configure_mkgridmap()
        if ret_val == 1:
            return 1
        elif ret_val == 2:
            error_level = 2

        #-------------------------------------------------------------------------------
        # Installs the Security Utilities
        #--------------------------------------------------------
            
        if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
            print "\nInstalling gLite Security Utilities               ",
            glib.printFailedMessage()
            return 1
                    
        print "\nInstalling gLite Security Utilities                        ",
        glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Configure pool accounts
        #-------------------------------------------------------------------------------
        
        ret_val = gsec.create_pool_accounts()
        if ret_val == 1:
            return 1
    
        #-------------------------------------------------------------------------------
        # Create the group map file /etc/grid-security/groupmapfile :
        #-------------------------------------------------------------------------------
    
        if gsec.createGroupmapfile():
            return 1
    
        #-------------------------------------------------------------------------------
        # Create vomsdir and check for VOMS certificates
        #-------------------------------------------------------------------------------
        
        if gsec.createVomsdir():
            return 1
    
        #-------------------------------------------------------------------------------
        # Create the $GLITE_LOCATION/etc/vomses file 
        #-------------------------------------------------------------------------------
    
        if gsec.createVomses():
            return 1

        #--------------------------------------------------------
        # Create all directories needed from glite_wms.conf file
        # Replacement of the glite-wms-parse-configuration script
        #--------------------------------------------------------

        if self.verbose:
            print "\nCreating or updating WMS specific directories and files ...  "
        
        # Replace token values in the template
        self.createWmsConfig()

        # Load WMS configuration file and update some values
        glite_wms_file = ClassAdsParser()
        
        glite_wms_file.verbose = verbose
        glite_wms_file.readParams(params['wms.config.file'])
        
        glite_wms_file.updateValue('LogMonitor', 'RemoveJobFiles', 'false')
        
        #    Patch 846
        glite_wms_file.addValue('LogMonitor', 'GlobusDownTimeout', '7200')
        glite_wms_file.updateValue('WorkloadManager','PipeDepth','200')
        glite_wms_file.addValue('WorkloadManager','WorkerThreads','20')
        glite_wms_file.updateValue('NetworkServer','BacklogSize','64')
        glite_wms_file.addValue('NetworkServer','ConnectionTimeout','300')

        
        if params.has_key('information.index.host'):
            glite_wms_file.updateValue('NetworkServer', 'II_Contact', '"' + params['information.index.host'] + '"')
        if params.has_key('information.index.port'):
            glite_wms_file.updateValue('NetworkServer', 'II_Port', params['information.index.port'])
            glite_wms_file.updateValue('NetworkServer', 'Gris_Port', params['information.index.port'])
        if params.has_key('information.index.base_dn'):
            glite_wms_file.updateValue('NetworkServer', 'II_DN', '"' + params['information.index.base_dn'] + '"')
            glite_wms_file.updateValue('NetworkServer', 'Gris_DN', '"' + params['information.index.base_dn'] + '"')
        glite_wms_file.updateValue('NetworkServer', 'ListMatchParadise', '"${GLITE_LOCATION_TMP}/MatchArea"')
        glite_wms_file.updateValue('NetworkServer', 'DLI_SI_CatalogTimeout', '60')
        glite_wms_file.updateValue('NetworkServer', 'SandboxStagingPath', '"${GLITE_LOCATION_VAR}/SandboxDir"')
        
        if params.has_key('wms.cemon.port'):
            if params['wms.cemon.port'] != "":
                glite_wms_file.addValue('WorkloadManager', 'CeMonitorAsynchPort', params['wms.cemon.port'])
            else:
                glite_wms_file.addValue('WorkloadManager', 'CeMonitorAsynchPort', '-1')
        else:
            glite_wms_file.addValue('WorkloadManager', 'CeMonitorAsynchPort', '-1')
        if params.has_key('wms.cemon.endpoints'):
            cemons = []
            for idx in xrange(len(params['wms.cemon.endpoints'])):
                if params['wms.cemon.endpoints'][idx] != "":
                       cemons.append('"' + params['wms.cemon.endpoints'][idx] + '"')
            if len(cemons) > 0:
                glite_wms_file.updateValue('WorkloadManager', 'CeMonitorServices', cemons)
        if params.has_key('disable.gris.purchasing'):
            glite_wms_file.addValue('WorkloadManager', 'DisablePurchasingFromGris', params['disable.gris.purchasing'])
        if params.has_key('enable.purchasing.from.rgma'):
            glite_wms_file.addValue('WorkloadManager', 'EnablePurchasingFromRgma', params['enable.purchasing.from.rgma'])
        if params.has_key('rgma.query.timeout'):
            glite_wms_file.addValue('WorkloadManager', 'RgmaQueryTimeout', params['rgma.query.timeout'])
        if params.has_key('rgma.consumer.ttl'):
            glite_wms_file.addValue('WorkloadManager', 'RgmaConsumerTtl', params['rgma.consumer.ttl'])
        if params.has_key('rgma.consumer.life.cycle'):
            glite_wms_file.addValue('WorkloadManager', 'RgmaConsumerLifeCycle', params['rgma.consumer.life.cycle'])
        if params.has_key('ism.rgma.purchasing.rate'):
            glite_wms_file.addValue('WorkloadManager', 'IsmRgmaPurchasingRate', params['ism.rgma.purchasing.rate'])
        if params.has_key('wms.si.service.type'):
            glite_wms_file.addValue('WorkloadManager', 'SiServiceName', '"%s"' % params['wms.si.service.type'])
        if params.has_key('wms.dli.service.type'):
            glite_wms_file.addValue('WorkloadManager', 'DliServiceName', '"%s"' % params['wms.dli.service.type'])
        if params.has_key('gpbox.hostname'):
            glite_wms_file.addValue('WorkloadManager', 'PboxHostName', '"%s"' % params['gpbox.hostname'])
        else:
            glite_wms_file.addValue('WorkloadManager', 'PboxHostName', '""')
        if params.has_key('gpbox.port.number'):
            glite_wms_file.addValue('WorkloadManager', 'PboxPortNum', '%s' % params['gpbox.port.number'])
        glite_wms_file.addValue('WorkloadManager', 'PboxSafeMode', 'false')

        glite_wms_file.addSection('WorkloadManagerProxy')
        glite_wms_file.updateValue('WorkloadManagerProxy', 'SandboxStagingPath', '"${GLITE_LOCATION_VAR}/SandboxDir"')
        glite_wms_file.updateValue('WorkloadManagerProxy', 'LogFile', '"${GLITE_LOCATION_LOG}/wmproxy.log"')
        glite_wms_file.updateValue('WorkloadManagerProxy', 'LogLevel', '6')
        glite_wms_file.updateValue('WorkloadManagerProxy', 'MaxInputSandboxSize', '10000000')
        glite_wms_file.updateValue('WorkloadManagerProxy', 'ListMatchRootPath', '"/tmp"')
        glite_wms_file.updateValue('WorkloadManagerProxy', 'LBProxy', 'true')
        glite_wms_file.updateValue('WorkloadManagerProxy', 'GridFTPPort', params['GSIWUFTPPORT'])
        if params.has_key('lb.server'):
            if params['lb.server'] != "":
                glite_wms_file.updateValue('WorkloadManagerProxy', 'LBServer', '"%s"' % params['lb.server'])
            else:
                glite_wms_file.deleteValue('WorkloadManagerProxy', 'LBServer')
        else:
            glite_wms_file.deleteValue('WorkloadManagerProxy', 'LBServer')
        glite_wms_file.updateValue('WorkloadManagerProxy', 'LBLocalLogger', '"%s"' % params['lb.locallogger'])
        if params.has_key('wmproxy.MinPerusalTimeInterval'):
            if params['wmproxy.MinPerusalTimeInterval'] != "":
                glite_wms_file.updateValue('WorkloadManagerProxy', 'MinPerusalTimeInterval', '%s' % params['wmproxy.MinPerusalTimeInterval'])
            else:
                glite_wms_file.updateValue('WorkloadManagerProxy', 'MinPerusalTimeInterval', '10')
        else:
            glite_wms_file.updateValue('WorkloadManagerProxy', 'MinPerusalTimeInterval', '10')

        glite_wms_file.addSection('WmsClient')

        glite_wms_file.writeParams(params['wms.config.file'])
        
        # Now read a flat list of params to create the directories
        glite_wms_file.fileName = params['wms.config.file']
        wms_params = glite_wms_file.getConfig()
        
        (uid,gid) = glib.get_user_info(params['glite.user.name'])
        
        glib.check_dir(os.environ['GLITE_LOCATION_VAR'], 01775, uid, gid)
        glib.check_dir(os.environ['GLITE_LOCATION_LOG'], 01775, uid, gid)

        for key in ['JobController.SubmitFileDir', 'JobController.OutputFileDir',\
                    'LogMonitor.CondorLogDir', 'LogMonitor.CondorLogRecycleDir', 'LogMonitor.MonitorInternalDir',\
                    'NetworkServer.SandboxStagingPath', 'NetworkServer.LogFile', 'NetworkServer.ListMatchParadise',\
                    'WorkloadManager.LogFile','WorkloadManagerProxy.SandboxStagingPath']:
            if key.endswith('File') :
                dir = os.path.dirname(wms_params[key])
            else:
                dir = wms_params[key]
            glib.check_dir(dir, 0775, uid, gid)

        for key in ['jobcontrol', 'logmonitor']:
            glib.check_dir(os.environ['GLITE_LOCATION_VAR'] + "/" + key, 0775, uid, gid)

        print "\nCreating or updating WMS specific directories and files   ",
        glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Configure Globus :
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "\nConfiguring Globus..."
        
        globus = Globus(params)
        globus.configure()

        print "\nConfiguring Globus                                        ",
        glib.printOkMessage()
    
        #-------------------------------------------------------------------
        # Condor
        #-------------------------------------------------------------------
    
        if self.verbose:
            print "\nConfiguring Condor ...                                 "
        
        # Check condor directory
        if not os.path.exists(params['condor.releasedir']):
            glib.printErrorMessage("The Condor package doesn't seem to be installed in %s. Configuration cannot continue" % params['condor.releasedir'])
            glib.printErrorMessage("Please verify that the correct path is specified in the condor.releasedir parameter")
            return 1

        (condor_uid,condor_gid) = glib.add_user(params['glite.user.name'],params['glite.user.group'])

        # Backup condor config files
        glib.check_dir(params['condor.localdir'],0755,condor_uid,condor_gid)
        condor_local_config = '%s/local.%s/condor_config.local' % (params['condor.releasedir'], glib.hostname)
        os.system("cp %s %s.1" % (condor_local_config,condor_local_config))
        os.system("cp %s %s.1" % (os.environ['CONDOR_CONFIG'],os.environ['CONDOR_CONFIG']))

        condor = Condor(params)
        condor.setValue("CONDOR_ADMIN",params['condor.condoradmin'])
        condor.setValue("RELEASE_DIR",params['condor.releasedir'])
        condor.setValue("CONDOR_HOST",os.environ['HOSTNAME'])
        condor.setValue("LOCAL_DIR",params['condor.localdir'])
        condor.setValue("DAEMON_LIST",params['condor.daemonlist'])
        condor.setValue("USERNAME",params['glite.user.name'])
        condor.setValue("SCHEDD_INTERVAL",params['condor.scheddinterval'])
        condor.setValue("CLASSAD_LIFETIME",params['condor.CLASSAD_LIFETIME'])
        condor.setValue("NEGOTIATOR_UPDATE_INTERVAL",params['condor.NEGOTIATOR_UPDATE_INTERVAL'])
        condor.setValue("MASTER_UPDATE_INTERVAL",params['condor.MASTER_UPDATE_INTERVAL'])
        condor.setValue("UPDATE_INTERVAL",params['condor.UPDATE_INTERVAL'])
        condor.setValue("NEGOTIATOR_INTERVAL",params['condor.NEGOTIATOR_INTERVAL'])
        condor.setValue("HIGHPORT",params['condor.HIGHPORT'])
        condor.setValue("LOWPORT",params['condor.LOWPORT'])
        condor.setValue("GRIDMANAGER_CHECKPROXY_INTERVAL","60")
        condor.setValue("CONDOR_IDS", "%s.%s" % (condor_uid,condor_gid))
        condor.setValue("GRIDMANAGER_DEBUG","D_COMMAND D_FULLDEBUG")
        condor.setValue("GRIDMANAGER_LOG","$(LOCAL_DIR)/log/GridmanagerLog.$(USERNAME)")
        condor.setValue("GRIDSHELL","$(RELEASE_DIR)/sbin/condor_starter")
        condor.setValue("MAX_C_GAHP_LOG","1000000")
        condor.setValue("C_GAHP_LOG","$(LOG)/CGAHPLog.$(USERNAME)")
        condor.setValue("C_GAHP_WORKER_THREAD_LOG","$(LOG)/CGAHPWorkerLog.$(USERNAME)")
        condor.setValue("CONDOR_GAHP","$(RELEASE_DIR)/sbin/condor_c-gahp")
        condor.setValue("CONDOR_JOB_POLL_INTERVAL",params['condor.blahpollinterval'])
        condor.setValue("SEC_DEFAULT_NEGOTIATION","REQUIRED")
        condor.setValue("SEC_DEFAULT_AUTHENTICATION","REQUIRED")
        condor.setValue("SEC_DEFAULT_AUTHENTICATION_METHODS","FS,GSI")
        condor.setValue("GSI_DAEMON_CERT","$ENV(GLITE_HOST_CERT)")
        condor.setValue("GSI_DAEMON_KEY","$ENV(GLITE_HOST_KEY)")
        condor.setValue("NEGOTIATOR_IGNORE_USER_PRIORITIES",params['condor.NEGOTIATOR_IGNORE_USER_PRIORITIES'])
        condor.setValue("DAGMAN_ALLOW_EVENTS","1")
        condor.setValue("GRIDMANAGER_MAX_JOBMANAGERS_PER_RESOURCE","20")
        condor.setValue("HOLD_JOB_IF_CREDENTIAL_EXPIRES","TRUE")
        condor.setValue("GRIDMANAGER_MAX_PENDING_REQUESTS","1000")
        condor.setValue("GRIDMANAGER_MINIMUM_PROXY_TIME","600")
        condor.setValue("GRIDMANAGER_MAX_SUBMITTED_JOBS_PER_RESOURCE","32000")
        condor.setValue("GRIDMANAGER_GAHP_CALL_TIMEOUT","900")
        condor.setValue("GRIDMANAGER_GLOBUS_COMMIT_TIMEOUT","3600")
        condor.setValue("GRID_MONITOR_HEARTBEAT_TIMEOUT","300")
        condor.setValue("GRID_MONITOR_RETRY_DURATION","31536000")
        condor.setValue("GLOBUS_GATEKEEPER_TIMEOUT","1800")
        condor.setValue("GT2_GAHP","%s/sbin/gahp_server" % params['LCG_LOCATION'])
        condor.setValue("GRID_MONITOR ","%s/sbin/grid_monitor.sh" % params['LCG_LOCATION'])
        
        if params['condor.ENABLE_GRID_MONITOR'] == "false":
            condor.setValue("ENABLE_GRID_MONITOR","False")
        else:
            condor.setValue("ENABLE_GRID_MONITOR","True")
       
        # Set permissions on condor local dir
        condor.condorDirs()

        # Copy local config file to condor local dir
        os.system("cp %s %s/condor_config.local" % (condor_local_config, params['condor.localdir']))

        print "\nConfiguring Condor                                        ",
        glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # WMS FTPD
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "\nConfiguring Grid FTP Server...                            "
        
        self.ftp.configure()
        # modify the ftpaccess file
        if not os.path.exists("%s/etc/ftpaccess" % os.environ['GLITE_LOCATION']):
            print "%s/etc/ftpaccess file does not exist" % os.environ['GLITE_LOCATION']
            return 1

        os.rename("%s/etc/ftpaccess" % os.environ['GLITE_LOCATION'], "%s/etc/ftpaccess.old" % os.environ['GLITE_LOCATION'])
        fin = open("%s/etc/ftpaccess.old" % os.environ['GLITE_LOCATION'],'r')
        fout = open("%s/etc/ftpaccess" % os.environ['GLITE_LOCATION'],'w')
        for line in fin.readlines():
            if line.find("magicgroup") > -1:
                fout.write("magicgroup %s all\n" % params['glite.user.group'])
            else:
                fout.write(line)
        fin.close()
        fout.close()
        os.remove("%s/etc/ftpaccess.old" % os.environ['GLITE_LOCATION'])

        print "\nConfiguring Grid FTP Server                               ",
        glib.printOkMessage()
    
        #-------------------------------------------------------------
        #  Locallogger daemon
        #------------------------------------------------------------
    
        if self.verbose:
            print "\nConfiguring the Local Logger Daemon...                    "
        
        (uid,gid) = glib.get_user_info(params['glite.user.name'])
        lb_cert_path = pwd.getpwnam(params['glite.user.name'])[5] + "/" + params['user.certificate.path']

        # Copy certificates
        glib.check_dir( lb_cert_path, 0755, uid, gid)
        os.system("cp %s %s %s/" % (params['host.certificate.file'], params['host.key.file'], lb_cert_path))
        glib.check_file("%s/hostcert.pem" % lb_cert_path, 0644, uid,gid)
        glib.check_file("%s/hostkey.pem" % lb_cert_path, 0400, uid,gid)

        print "\nConfiguring the Local Logger Daemon                       ",
        glib.printOkMessage()
           
        #-------------------------------------------------------------
        #  Configuring WMProxy...
        #------------------------------------------------------------
        
        if self.verbose:
            print "\nConfiguring the WMProxy web service..."

        # Set the excutable bit
        os.system("/bin/chmod +s %s/bin/glite_wms_wmproxy_dirmanager" % os.environ['GLITE_LOCATION'])

        # Edit the authorization file
        if self.verbose:
            print "Creating the WMProxy authorization file                  ",

        try:

            impl = getDOMImplementation()
            doc = impl.createDocument(None, "gacl", None)
            root = xpath.Evaluate("/gacl", doc.documentElement)[0]
            root.setAttribute('version','0.0.1')
            for vo in glib.getVoList():
                entryElement = doc.createElement("entry")
                root.appendChild(entryElement)
                vomsElement = doc.createElement("voms")
                entryElement.appendChild(vomsElement)
                fqanElement = doc.createElement("fqan")
                vomsElement.appendChild(fqanElement)
                fqanElementText = doc.createTextNode("%s/Role=NULL" % vo)
                fqanElement.appendChild(fqanElementText)
                allowElement = doc.createElement("allow")
                entryElement.appendChild(allowElement)
                execElement = doc.createElement("exec")
                allowElement.appendChild(execElement)
    
            file = open("%s/etc/glite_wms_wmproxy.gacl" % os.environ['GLITE_LOCATION'], "w")
            PrettyPrint(doc.documentElement, stream=file)
            file.close()
            
            doc.unlink
            
        except Exception, msg:

            if self.verbose:
                glib.printFailedMessage()
            glib.printErrorMessage("Could not create WMProxy authorization file")
            glib.printErrorMessage("(%s)" % msg)
            return 1

        if self.verbose:
            glib.printOkMessage()
        
        # Edit the httpd/GridSite configuration file and check values inside
        input = open("%s/etc/glite_wms_wmproxy_httpd.conf.template" % os.environ['GLITE_LOCATION'],'r')
        output = open("%s/etc/glite_wms_wmproxy_httpd.conf.template.1" % os.environ['GLITE_LOCATION'],'w')
        for line in input:
            if line.find('</VirtualHost>') == -1:
                output.write(line)
            else:
                output.write(' PassEnv GLITE_LOCATION\n')
                output.write(' PassEnv GLITE_LOCATION_VAR\n')
                output.write(' PassEnv GLITE_LOCATION_LOG\n')
                output.write(' PassEnv GLITE_LOCATION_TMP\n')
                output.write('\n')
                output.write('</VirtualHost>\n')
                
        input.close()
        output.close()

        os.rename("%s/etc/glite_wms_wmproxy_httpd.conf.template.1" % os.environ['GLITE_LOCATION'], \
            "%s/etc/glite_wms_wmproxy_httpd.conf.template" % os.environ['GLITE_LOCATION'])
                            
        input = open("%s/etc/glite_wms_wmproxy_httpd.conf.template" % os.environ['GLITE_LOCATION'],'r')
        for line in input:
            if line.find("-initial-env GLITE_LOCATION_VAR") != -1:
                input.close()
                break
        else:
            input = open("%s/etc/glite_wms_wmproxy_httpd.conf.template" % os.environ['GLITE_LOCATION'],'r')
            output = open("%s/etc/glite_wms_wmproxy_httpd.conf.template.1" % os.environ['GLITE_LOCATION'],'w')
            for line in input:
                if line.find("FastCgiConfig") != -1:
                    output.write(line.strip() + " -initial-env GLITE_LOCATION_VAR -initial-env GLITE_LOCATION_TMP -initial-env GLITE_LOCATION_LOG\n")
                else:
                    output.write(line)
                    
            input.close()
            output.close()

            os.rename("%s/etc/glite_wms_wmproxy_httpd.conf.template.1" % os.environ['GLITE_LOCATION'], \
                "%s/etc/glite_wms_wmproxy_httpd.conf.template" % os.environ['GLITE_LOCATION'])

        input = open("%s/etc/glite_wms_wmproxy_httpd.conf.template" % os.environ['GLITE_LOCATION'],'r')
        output = open("%s/etc/glite_wms_wmproxy_httpd.conf" % os.environ['GLITE_LOCATION'],'w')
        for line in input:
            if line.strip().startswith('ServerName'):
                output.write('ServerName %s' % os.environ['HOSTNAME'])
            else:
                output.write(line)
                
        output.close()
        input.close()

        # Make sure dirs/files have correct ownership and permissions
        uid,gid = glib.get_user_info(params['glite.user.name'])
        glib.check_dir("%s/SandboxDir" % os.environ['GLITE_LOCATION_VAR'], 0773, uid, gid)
        #glib.check_dir("%s/proxycache" % os.environ['GLITE_LOCATION_TMP'], 0771, uid, gid)
        glib.check_dir("%s/proxycache" % os.environ['GLITE_LOCATION_VAR'], 0771, uid, gid)
        
        glib.check_file("%s/spool/glite-renewd" % os.environ['GLITE_LOCATION_VAR'],0773,uid,gid)
        os.system("chmod u+s %s/bin/glite_wms_wmproxy_dirmanager" % os.environ['GLITE_LOCATION'])
       
        #-------------------------------------------------------------
        #  Configuring LBProxy...
        #------------------------------------------------------------
    
        if self.verbose:
            print "\nConfiguring the LBProxy web service..."
            
        # start MySQL
        self.mysql.stop()
        time.sleep(5)
        self.mysql.start()                

        if not os.path.exists('/tmp/mysql.sock'):
            os.symlink('/var/lib/mysql/mysql.sock', '/tmp/mysql.sock')

        # ------------------------------------------------------------
        # Check configuration of MySQL
        # ------------------------------------------------------------
        
        self.mysql_root_password = params['mysql.root.password']
        if not params.has_key('set.mysql.root.password'):
            params['set.mysql.root.password'] = 'false'
        setempty = params['set.mysql.root.password']
        if self.mysql.checkMySQLConfiguration(self.mysql_root_password,setempty):
            return 1
        
        # Create the MySQL database
        print "\nCreate/Verify the %s database" % params['lb.proxy.database.name']
        
        # Check if database exists
        if self.mysql.existsDB(params['lb.proxy.database.name'],self.mysql_root_password) != 0:

            # Create database
            print ('\n==> Creating MySQL %s database\n' % params['lb.proxy.database.name'])
    
            if os.path.exists('/bin/rm /tmp/mysql_ct'):
                os.remove('/tmp/mysql_ct')
            
            file = open('/tmp/mysql_ct', 'w')
            text = ['CREATE DATABASE %s;\n' % params['lb.proxy.database.name'], 
                       'GRANT ALL PRIVILEGES ON %s.* TO %s@localhost IDENTIFIED BY "";\n' \
                       % (params['lb.proxy.database.name'],params['lb.proxy.database.username']),
                       'USE %s;\n' % params['lb.proxy.database.name'],
                       '\. %s/etc/glite-lb-dbsetup-proxy.sql\n' % os.environ['GLITE_LOCATION']]
    
            file.writelines(text)
            file.close()
            os.system('/usr/bin/mysql -p%s < /tmp/mysql_ct' % self.mysql_root_password)
            os.system('/bin/rm /tmp/mysql_ct')
            
        else:
            print "\n==> MySQL database %s already exist\n" % params['lb.proxy.database.name']
            
        #-------------------------------------------------------------
        #  Configuring cron jobs...
        #------------------------------------------------------------
    
        print "\nInstalling Purge Storage cron job as"
        print "    /etc/cron.d/glite-wms-purger.cron                     ",
        
        crontab = "/etc/cron.d/glite-wms-purger.cron"
        cronjobH = "0 */1 * * mon-sat %s . %s ; $GLITE_LOCATION/sbin/glite-wms-purgeStorage -l $GLITE_LOCATION_LOG/glite-wms-purgeStorage.log -t 604800 -a 40 > /dev/null" \
            % (glib.getInstallerExportFile(), os.environ['GLITE_USER'])
        cronjobW = "0 */4 * * sun %s . %s ; $GLITE_LOCATION/sbin/glite-wms-purgeStorage -l $GLITE_LOCATION_LOG/glite-wms-purgeStorage.log -t 604800 > /dev/null" \
            % (glib.getInstallerExportFile(), os.environ['GLITE_USER'])

        if not os.path.exists(crontab):
            os.system("/bin/touch %s" % crontab)
        os.chmod(crontab,0644)
        f = open(crontab,'w')
        f.write("HOME=/\n")
        f.write("MAILTO=%s\n\n" % params['cron.mailto'])
        f.write("# Execute the 'purger' command at every day except on Sunday with a frequency of one hour\n")
        f.write("# if and only if the percentage of allocated blocks is greater than 40%\n")
        f.write(cronjobH)
        f.write("\n\n# Execute the 'purger' command at 4:00 AM, 8:00 AM, 12:00 noon, 4:00 PM,\n")
        f.write("# and 8:00 PM (0 */4) on each Sunday (sun).\n")
        f.write(cronjobW)
        f.write("\n")
        f.close()
    
        glib.printOkMessage()
    
        print "\nInstalling Network Server proxy renewal cron job as"
        print "    /etc/cron.d/glite-wms-ns-proxy.cron                   ",
        
        crontab = "/etc/cron.d/glite-wms-ns-proxy.cron"
        cronjob = "5 2,8,14,20 * * * root . %s ; $GLITE_LOCATION/etc/init.d/glite-wms-ns proxy > /dev/null" \
            % glib.getInstallerExportFile()

        if not os.path.exists(crontab):
            os.system("/bin/touch %s" % crontab)
        os.chmod(crontab,0644)
        f = open(crontab,'w')
        f.write("HOME=/\n")
        f.write("MAILTO=%s\n\n" % params['cron.mailto'])
        f.write(cronjob)
        f.write("\n")
        f.close()
    
        glib.printOkMessage()

        print "\nInstalling WMSProxy Purge Proxy Cache cron job as"
        print "    /etc/cron.d/glite-wms-wmproxy-purge-proxycache.cron   ",
        
        crontab = "/etc/cron.d/glite-wms-wmproxy-purge-proxycache.cron"
        cronjob = "0 */6 * * * root . %s ; $GLITE_LOCATION/bin/glite_wms_wmproxy_purge_proxycache $GLITE_LOCATION_VAR/proxycache > $GLITE_LOCATION_LOG/glite_wms_wmproxy_purge_proxycache.log" \
            % glib.getInstallerExportFile()

        if not os.path.exists(crontab):
            os.system("/bin/touch %s" % crontab)
        os.chmod(crontab,0644)
        f = open(crontab,'w')
        f.write("HOME=/\n")
        f.write("MAILTO=%s\n\n" % params['cron.mailto'])
        f.write(cronjob)
        f.write("\n")
        f.close()
    
        glib.printOkMessage()

        #-------------------------------------------------------------------
        # RGMA Client
        #-------------------------------------------------------------------
        cmd = "python %s/glite-rgma-client-config.py --configure --subservice" % glib.getScriptPath()
        if os.system(cmd):
            glib.printErrorMessage("Failed to configure R-GMA Client")
            return 1
          
        #------------------------------------------------------------
        # configure service discovery
        # needed to locate Fireman catalogs during matchmaking
        #------------------------------------------------------------

        if serviceDiscovery.configure(params,glib):
            return 1
        
        #-------------------------------------------------------------------
        # RGMA servicetool: configure servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
        
            # Create Local Logger instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['locallogger.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['locallogger.serviceName'])
            servicetoolInstance.setServiceType(params['locallogger.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['locallogger.statusScript'])
            servicetoolInstance.setEndpoint(params['locallogger.endpoint'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create ProxyRenewal instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['proxyrenewal.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['proxyrenewal.serviceName'])
            servicetoolInstance.setServiceType(params['proxyrenewal.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['proxyrenewal.statusScript'])
            servicetoolInstance.setEndpoint(params['proxyrenewal.endpoint'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create LogMonitor instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['logmonitor.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['logmonitor.serviceName'])
            servicetoolInstance.setServiceType(params['logmonitor.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['logmonitor.statusScript'])
            servicetoolInstance.setEndpoint(params['logmonitor.endpoint'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create JobController instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['jobcontroller.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['jobcontroller.serviceName'])
            servicetoolInstance.setServiceType(params['jobcontroller.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['jobcontroller.statusScript'])
            servicetoolInstance.setEndpoint(params['jobcontroller.endpoint'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create NetworkServer instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['networkserver.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['networkserver.serviceName'])
            servicetoolInstance.setServiceType(params['networkserver.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['networkserver.statusScript'])
            servicetoolInstance.setEndpoint(params['networkserver.endpoint'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create WorkloadManager instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['workloadmanager.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['workloadmanager.serviceName'])
            servicetoolInstance.setServiceType(params['workloadmanager.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['workloadmanager.statusScript'])
            servicetoolInstance.setEndpoint(params['workloadmanager.endpoint'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create WMProxy instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['wmproxy.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['wmproxy.serviceName'])
            servicetoolInstance.setServiceType(params['wmproxy.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['wmproxy.statusScript'])
            servicetoolInstance.setEndpoint(params['wmproxy.endpoint'])
            servicetoolInstance.setSemanticsUrl(params['wmproxy.serviceSemanticsURL'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create LBProxy instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['lbproxy.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['lbproxy.serviceName'])
            servicetoolInstance.setServiceType(params['lbproxy.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['lbproxy.statusScript'])
            servicetoolInstance.setEndpoint(params['lbproxy.endpoint'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Configure servicetool
            if rgmaServicetool.configure(glib):
                # error in configuring servicetool
                return 1
            
        #-------------------------------------------------------------------
        # Set R-GMA client authentication properties for wms user
        #-------------------------------------------------------------------
        path = "%s/etc/rgma/ClientAuthentication-wms.props" % os.environ['GLITE_LOCATION']

        try:
            file = open(path, 'w')
            file.write('sslCertFile=%s\n' % os.environ['GLITE_HOST_CERT'])
            file.write('sslKey=%s\n' % os.environ['GLITE_HOST_KEY'])
            file.write('crlFiles=%s/*.r0\n' % params['ca.certificates.dir'])
            file.write('sslCAFiles=%s/*.0\n' % params['ca.certificates.dir'])
            file.write('#gridProxyFile=/tmp/grid-user-proxy.pem\n')
            file.close()
        except IOError, ex:
            glib.printFailedMessage()
            if ex.strerror == 'Permission denied':
                glib.printErrorMessage("File '%s' is not writable" % path)
                return 1
            else:
                glib.printErrorMessage(ex.strerror)
                return 1

        return error_level
            
        
    def replaceToken(self,line,token,value):
        if line.find("${"+token+"}") > -1:
            line = string.replace(line,"${"+token+"}",value)
        return line

    def redefineValue(self,line,name,value):
        if line.find(name) > -1:
            line = string.split(line,"=")[0] + " = " + value + ";\n"
        return line

                
    def createWmsConfig(self):

        inFile = open("%s.template" % params['wms.config.file'],"r")
        outFile = open(params['wms.config.file'],"w")
        for line in inFile.readlines():
            if line.find("NumberOfWorkerThreads") == -1:
                if line.find("$") != -1:
                    line = self.replaceToken(line,"GLITE_WMS_USER","${GLITE_USER}")
                    line = string.replace(line,"${GLITE_WMS_LOCATION_VAR}/log/","${GLITE_LOCATION_LOG}/")
                    line = self.replaceToken(line,"GLITE_WMS_LOCATION_VAR","${GLITE_LOCATION_VAR}")
                    line = self.replaceToken(line,"GLITE_WMS_TMP","${GLITE_LOCATION_VAR}")
                    line = self.replaceToken(line,"GLITE_WMS_LOCATION","${GLITE_LOCATION}")
                    line = self.replaceToken(line,"CONDORG_INSTALL_PATH","${CONDORG_INSTALL_PATH}")
                    line = string.replace(line,"${GLITE_LOCATION_VAR}/log/","${GLITE_LOCATION_LOG}/")
                outFile.write(line)
        inFile.close()
        outFile.close()

        inFile = open("%s/etc/glite_wms_wmproxy_httpd.conf.template" % os.environ['GLITE_LOCATION'],'r')
        outFile = open("%s/etc/glite_wms_wmproxy_httpd.conf.template.1" % os.environ['GLITE_LOCATION'],"w")
        for line in inFile.readlines():
            if line.find("$") != -1:
                line = self.replaceToken(line,"GLITE_WMS_USER","${GLITE_USER}")
                line = string.replace(line,"${GLITE_WMS_LOCATION_VAR}/log/","${GLITE_LOCATION_LOG}/")
                line = self.replaceToken(line,"GLITE_WMS_LOCATION_VAR","${GLITE_LOCATION_VAR}")
                line = self.replaceToken(line,"GLITE_WMS_TMP","${GLITE_LOCATION_VAR}")
                line = self.replaceToken(line,"GLITE_WMS_LOCATION","${GLITE_LOCATION}")
                line = string.replace(line,"${GLITE_LOCATION_VAR}/log/","${GLITE_LOCATION_LOG}/")
            outFile.write(line)
        inFile.close()
        outFile.close()
        os.rename("%s/etc/glite_wms_wmproxy_httpd.conf.template.1" % os.environ['GLITE_LOCATION'], \
                  "%s/etc/glite_wms_wmproxy_httpd.conf.template" % os.environ['GLITE_LOCATION'])

        # glite-wms-vars.sh
        outFile = open("%s/etc/glite-wms-vars.sh" % params['GLITE_LOCATION'],"w")
        outFile.write("#!/bin/sh\n")
        outFile.write("GLOBUS_LOCATION=${GLOBUS_LOCATION:-/opt/globus}\n")
        outFile.write("GLITE_WMS_LOCATION_VAR=${GLITE_LOCATION_VAR:-/var}\n")
        outFile.write("GLITE_WMS_USER=${GLITE_USER:-%s}\n" % params['glite.user.name'])
        outFile.write("GLITE_WMS_GROUP=${GLITE_WMS_GROUP:-%s}\n" % params['glite.user.group'])
        outFile.write("GLITE_WMS_TMP=${GLITE_LOCATION_VAR:-/var/glite}\n")
        outFile.write("GLITE_WMS_PATH=${GLITE_LOCATION}/bin:${GLITE_LOCATION}/sbin\n")
        outFile.write("GLITE_WMS_LIBRARY_PATH=${GLOBUS_LOCATION}/lib:${GLITE_LOCATION}/lib\n")
        outFile.close()

        # glite-wms-vars.csh
        outFile = open("%s/etc/glite-wms-vars.csh" % params['GLITE_LOCATION'],"w")
        outFile.write("#!/bin/csh\n")
        outFile.write("if ( ! $?GLOBUS_LOCATION) then\n")
        outFile.write('    setenv GLOBUS_LOCATION "/opt/globus"\n')
        outFile.write("endif\n\n")
        outFile.write("if ( $?GLITE_WMS_USER) then\n")
        outFile.write("  setenv GLITE_WMS_USER %s\n" % params['glite.user.name'])
        outFile.write("endif\n\n")
        outFile.write("if ( $?GLITE_WMS_GROUP) then\n")
        outFile.write("  setenv GLITE_WMS_GROUP %s\n" % params['glite.user.group'])
        outFile.write("endif\n\n")
        outFile.write("if ( $?GLITE_WMS_LOCATION_VAR) then\n")
        outFile.write("  setenv GLITE_WMS_LOCATION_VAR ${GLITE_LOCATION_VAR}\n")
        outFile.write("endif\n\n")
        outFile.write("if ( $?GLITE_WMS_TMP) then\n")
        outFile.write("  setenv GLITE_WMS_TMP ${GLITE_LOCATION_VAR}\n")
        outFile.write("endif\n\n")
        outFile.write('setenv GLITE_WMS_PATH ${GLITE_LOCATION}"/bin:"${GLITE_LOCATION}"/sbin"\n')
        outFile.write('setenv GLITE_WMS_LIBRARY_PATH ${GLOBUS_LOCATION}"/lib:"${GLITE_LOCATION}"/lib"\n')
        outFile.close()

def loadDefaults(params):

    params['GLITE_LOCATION'] = "/opt/glite"
    params['mkgridmap.script'] = "%s/sbin/glite-mkgridmap" % params['GLITE_LOCATION']
    params['mkgridmap.conf'] = "%s/etc/glite-mkgridmap.conf" % params['GLITE_LOCATION']
    params['host.gridmap.dir'] = "/etc/grid-security/gridmapdir"
    params['condor.NEGOTIATOR_IGNORE_USER_PRIORITIES'] = "True"
    params['mysql.root.password'] = ""
    params['mysql.max_allowed_packet'] = "17M"
    params['lb.proxy.database.name'] = "lbproxy"
    params['lb.proxy.database.username'] = "lbserver"
    params['wms.si.service.type'] = "org.glite.SEIndex"
    params['wms.dli.service.type'] = "data-location-interface"
    
    params['locallogger.serviceName'] = 'LB Local Logger at %s' % glib.fq_hostname
    params['locallogger.serviceType'] = 'org.glite.lb.locallogger'
    params['locallogger.statusScript'] = '%s/etc/init.d/glite-lb-locallogger status' % params['GLITE_LOCATION']
    params['locallogger.endpoint'] = 'not available'
    
    params['proxyrenewal.serviceName'] = 'Proxy Renewal service at %s' % glib.fq_hostname
    params['proxyrenewal.serviceType'] = 'org.glite.security.proxyrenewal'
    params['proxyrenewal.statusScript'] = '%s/etc/init.d/glite-proxy-renewald status' % params['GLITE_LOCATION']
    params['proxyrenewal.endpoint'] = 'not available'
    
    params['logmonitor.serviceName'] = 'Log Monitor service at %s' % glib.fq_hostname
    params['logmonitor.serviceType'] = 'org.glite.wms.logmonitor'
    params['logmonitor.statusScript'] = '%s/etc/init.d/glite-wms-lm status' % params['GLITE_LOCATION']
    params['logmonitor.endpoint'] = 'not available'
    
    params['jobcontroller.serviceName'] = 'Job Controller service at %s' % glib.fq_hostname
    params['jobcontroller.serviceType'] = 'org.glite.wms.jobcontroller'
    params['jobcontroller.statusScript'] = '%s/etc/init.d/glite-wms-jc status' % params['GLITE_LOCATION']
    params['jobcontroller.endpoint'] = 'not available'
    
    params['networkserver.serviceName'] = 'Network Server service at %s' % glib.fq_hostname
    params['networkserver.serviceType'] = 'org.glite.wms.networkserver'
    params['networkserver.statusScript'] = '%s/etc/init.d/glite-wms-ns status' % params['GLITE_LOCATION']
    params['networkserver.endpoint'] = 'not available'
    
    params['workloadmanager.serviceName'] = 'Workload Manager service at %s' % glib.fq_hostname
    params['workloadmanager.serviceType'] = 'org.glite.wms.workloadmanager'
    params['workloadmanager.statusScript'] = '%s/etc/init.d/glite-wms-wm status' % params['GLITE_LOCATION']
    params['workloadmanager.endpoint'] = 'not available'
    
    params['wmproxy.serviceName'] = 'WMProxy service at %s' % glib.fq_hostname
    params['wmproxy.serviceType'] = 'org.glite.wms'
    params['wmproxy.statusScript'] = '/sbin/service httpd status'
    params['wmproxy.endpoint'] = 'https://%s:7443/glite_wms_wmproxy_server' % glib.fq_hostname
    params['wmproxy.serviceSemanticsURL'] = 'http://egee-jra1-wm.mi.infn.it/egee-jra1-wm/wmproxy'
    
    params['lbproxy.serviceName'] = 'LBProxy service at %s' % glib.fq_hostname
    params['lbproxy.serviceType'] = 'org.glite.wms.lbproxy'
    params['lbproxy.statusScript'] = '%s/etc/init.d/glite-lb-proxy status' % params['GLITE_LOCATION']
    params['lbproxy.endpoint'] = 'not available'
    
def set_env(params):

    # gLite env
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)
    
    # Globus env
    glib.export('GLOBUS_LOCATION','/opt/globus')
    glib.export('GPT_LOCATION','/opt/gpt')
    
    # Condor env
    params['condor.user'] = params['glite.user.name']
    glib.export('CONDORG_INSTALL_PATH',params['condor.releasedir'])
    params['CONDOR_CONFIG'] = '%s/etc/condor_config' % params['condor.releasedir']
    glib.export('CONDOR_CONFIG', params['CONDOR_CONFIG'])

    # Security env
    glib.export('X509_CERT_DIR','/etc/grid-security/certificates')
    glib.export('GRID_FUNCTIONS','/etc/grid-functions')

    # Path
    glib.addEnvPath('PATH', os.environ['CONDORG_INSTALL_PATH'] + '/bin')
    glib.addEnvPath('PATH', os.environ['GLOBUS_LOCATION'] + '/bin')
    glib.addEnvPath('PATH', "/opt/condor-c/libexec/glite")
    glib.addEnvPath('PATH', os.environ['GLITE_LOCATION'] + '/externals/bin')
    glib.addEnvPath('PATH', os.environ['GLITE_LOCATION'] + '/bin')

    # Library path
    glib.addEnvPath('LD_LIBRARY_PATH', os.environ['GLOBUS_LOCATION'] + '/lib')
    glib.addEnvPath('LD_LIBRARY_PATH', os.environ['GLITE_LOCATION'] + '/externals/lib')
    glib.addEnvPath('LD_LIBRARY_PATH', os.environ['GLITE_LOCATION'] + '/lib')
    glib.addEnvPath('LD_LIBRARY_PATH', os.environ['CONDORG_INSTALL_PATH'] + '/lib')
    
    #### Nasty workaround for bug #13855
    glib.addEnvPath('LD_LIBRARY_PATH', os.environ['GLITE_LOCATION'])
    #### Nasty workaround for bug #13855

    # WMS env
    glib.export('GLITE_WMS_LOCATION',os.environ['GLITE_LOCATION'])
    glib.export('GLITE_WMS_LOCATION_VAR',os.environ['GLITE_LOCATION_VAR'])
    glib.export('GLITE_WMS_USER', params['glite.user.name'])
    glib.export('GLITE_WMS_GROUP', params['glite.user.group'])
    glib.export('GLITE_WMS_TMP', os.environ['GLITE_LOCATION_VAR'])
    glib.export('GLITE_WMS_LOCATION_VAR', os.environ['GLITE_LOCATION_VAR'])
    glib.export('GLITE_WMS_PATH', '%s/bin:%s/sbin' \
        % (os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION']))
    glib.export('GLITE_WMS_LIBRARY_PATH','%s/lib:%s/lib' \
        % (os.environ['GLITE_LOCATION'],os.environ['GLOBUS_LOCATION']))
    glib.export('GRIDMAP', params['host.gridmapfile'])
    glib.export('GRIDMAPDIR', params['host.gridmap.dir'])
    
    glib.export('GLITE_WMS_QUERY_TIMEOUT', '300')
    # LB and proxy renewal env
    glib.export('GLITE_USER',params['glite.user.name'])
    if verbose:
        print "Checking local logger user"
    uid,gid = glib.add_user(os.environ['GLITE_USER'],params['glite.user.group'])
        
    glib.export('GLITE_HOST_CERT',"%s/%s/hostcert.pem" \
        % (pwd.getpwnam(os.environ['GLITE_USER'])[5],params['user.certificate.path']))
    glib.export('GLITE_HOST_KEY',"%s/%s/hostkey.pem" \
        % (pwd.getpwnam(os.environ['GLITE_USER'])[5],params['user.certificate.path']))

    if params.has_key('BDII_HOST'):
        glib.export('LCG_GFAL_INFOSYS',"%s:2170" % params['BDII_HOST'])
        
    if not params.has_key('X509_VOMS_DIR'):
        params['X509_VOMS_DIR'] = '/etc/grid-security/vomsdir'
    glib.export('X509_VOMS_DIR', params['X509_VOMS_DIR'])
    glib.export('X509_VOMS_DIR', params['X509_VOMS_DIR'])
    glib.check_dir(params['X509_VOMS_DIR'])
    if os.environ.has_key('GLOBUS_TCP_PORT_RANGE'):
        #params['GLOBUS_TCP_PORT_RANGE'] = '"%s"' % os.environ['GLOBUS_TCP_PORT_RANGE']
        glib.export('GLOBUS_TCP_PORT_RANGE', os.environ['GLOBUS_TCP_PORT_RANGE'])
        low, high = os.environ['GLOBUS_TCP_PORT_RANGE'].split(' ')
        glib.export('MYPROXY_TCP_PORT_RANGE', "%s,%s" % (low, high))
    # R-GMA env
    glib.export('JAVA_HOME')
    glib.addEnvPath('PATH', os.environ['JAVA_HOME'] + '/bin')
    glib.export('TRUSTFILE',"%s/etc/rgma/ClientAuthentication-wms.props" % os.environ['GLITE_LOCATION'])

    # MANPATH
    glib.addEnvPath('MANPATH', os.environ['GLOBUS_LOCATION'] + '/man')
    glib.addEnvPath('MANPATH', os.environ['GLITE_LOCATION'] + '/share/man')

    # Perl
    glib.addEnvPath("PERL5LIB", "%s/lib/perl:%s/lib/perl5" % \
                     (os.environ['GPT_LOCATION'], os.environ['GLITE_LOCATION']))
    
    # Set environment
    glib.setUserEnv()

#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid()==0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = GLib()
    
    # Load parameters
    loadDefaults(params)
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
        
    if glib.loadConfiguration(["%s/../glite-wms.cfg.xml" % glib.getScriptPath(), \
            "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath(), \
            "%s/../glite-service-discovery.cfg.xml" % glib.getScriptPath()],params):
        print "An error occurred while configuring the service"
        sys.exit(1)

    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    glib.verbose = verbose
    
    # For the time being the standard grid-mapfile is needed on WMS
    params['install.mkgridmap.cron'] = 'true'

    # Set up the environment
    set_env(params)
          
    #Instantiate grdSecurity classes
    gsec = gridSecurity(glib)
    gsec.setVerbosity(verbose)
    gsec.params = params

    # Instantiate the service classes
    service = glite_wms(params)
    service.verbose = verbose

    # Instantiate the service discovery class
    serviceDiscovery = ServiceDiscovery()
    serviceDiscovery.verbose = verbose
        
    # Instantiate the rgma servicetool class
    rgmaServicetool = gliteRgmaServicetool()
    rgmaServicetool.verbose = verbose        
    
    # Command line opts if any
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'configure', 'start', 'stop', 'status', 'startservice=', 'stopservice=', 'siteconfig='])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # Check cli options
    for o, a in opts:
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            print
            rgmaServicetool.showServices()
            sys.exit(0)

        if o == "--configure":

            service.copyright()
            service.showVersion()
            service.banner()
            
            # Check certificates
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s service" \
                            % service.friendly_name
                        sys.exit(1)    
                
            # Display the parameters if verbose output
            if verbose: 
                glib.print_params(params)
                
            # Stop all services
            glib.printInfoMessage("\n\nStopping all running WMS services...")
            service.stop()
            
            # Configure the service
            return_result = service.configure()

            if return_result == 0:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running WMS services...")
                service.stop()
                
                glib.registerService()

                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
                print "\nBefore using the %s services make sure that" % service.friendly_name
                print "the host certificate of each used VOMS server is installed in"
                print "%s\n" % os.environ['X509_VOMS_DIR']
                print "You can now start the service using the --start option of this script\n\n"

                sys.exit(0)

            elif return_result == 2:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running WMS services...")
                service.stop()
                
                print "\n\nThe %s configuration was completed,\n" % service.friendly_name
                print "but warnings were issued. Please revise them and re-run the script\n"
                print "or configure WMS manually\n"

                sys.exit(2)
            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s" \
                    % service.friendly_name

                sys.exit(1)
            
        if o == "--start":
            if service.start() == 0:
                print "\n\nThe %s was successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while starting the %s" \
                    % service.friendly_name
                sys.exit(1)
                
        if o == "--stop":
            if service.stop() == 0:
                print "\n\nThe %s was successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while stopping the %s" \
                    % service.friendly_name
                sys.exit(1)
                
        if o == "--startservice":
            if service.start(a) == 0:
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while starting the %s service " % a
                sys.exit(1)
                
        if o == "--stopservice":
            if service.stop(a) == 0:
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while stopping the %s service" % a
                sys.exit(1)
                
        if o == "--status":
            retval = service.status()
            sys.exit(retval)

